precision highp float;
varying highp vec2 v_coordinate;

uniform sampler2D u_texture;

uniform highp vec2 u_singleStepOffset;
uniform highp vec4 u_params;
uniform highp float u_brightness;
uniform int u_has_beauty;

const highp vec3 W = vec3(0.299, 0.587, 0.114);
const highp mat3 saturateMatrix = mat3(
                                    1.1102, -0.0598, -0.061,
                                    -0.0774, 1.0826, -0.1186,
                                    -0.0228, -0.0228, 1.1772);
highp vec2 blurCoordinates[24];

highp float hardLight(highp float color) {
 if (color <= 0.5)
     color = color * color * 2.0;
 else
     color = 1.0 - ((1.0 - color)*(1.0 - color) * 2.0);
 return color;
}

void main(){
     highp vec3 centralColor = texture2D(u_texture, v_coordinate).rgb;
     if (u_has_beauty == 1){
         blurCoordinates[0] = v_coordinate.xy + u_singleStepOffset * vec2(0.0, -10.0);
         blurCoordinates[1] = v_coordinate.xy + u_singleStepOffset * vec2(0.0, 10.0);
         blurCoordinates[2] = v_coordinate.xy + u_singleStepOffset * vec2(-10.0, 0.0);
         blurCoordinates[3] = v_coordinate.xy + u_singleStepOffset * vec2(10.0, 0.0);
         blurCoordinates[4] = v_coordinate.xy + u_singleStepOffset * vec2(5.0, -8.0);
         blurCoordinates[5] = v_coordinate.xy + u_singleStepOffset * vec2(5.0, 8.0);
         blurCoordinates[6] = v_coordinate.xy + u_singleStepOffset * vec2(-5.0, 8.0);
         blurCoordinates[7] = v_coordinate.xy + u_singleStepOffset * vec2(-5.0, -8.0);
         blurCoordinates[8] = v_coordinate.xy + u_singleStepOffset * vec2(8.0, -5.0);
         blurCoordinates[9] = v_coordinate.xy + u_singleStepOffset * vec2(8.0, 5.0);
         blurCoordinates[10] = v_coordinate.xy + u_singleStepOffset * vec2(-8.0, 5.0);
         blurCoordinates[11] = v_coordinate.xy + u_singleStepOffset * vec2(-8.0, -5.0);
         blurCoordinates[12] = v_coordinate.xy + u_singleStepOffset * vec2(0.0, -6.0);
         blurCoordinates[13] = v_coordinate.xy + u_singleStepOffset * vec2(0.0, 6.0);
         blurCoordinates[14] = v_coordinate.xy + u_singleStepOffset * vec2(6.0, 0.0);
         blurCoordinates[15] = v_coordinate.xy + u_singleStepOffset * vec2(-6.0, 0.0);
         blurCoordinates[16] = v_coordinate.xy + u_singleStepOffset * vec2(-4.0, -4.0);
         blurCoordinates[17] = v_coordinate.xy + u_singleStepOffset * vec2(-4.0, 4.0);
         blurCoordinates[18] = v_coordinate.xy + u_singleStepOffset * vec2(4.0, -4.0);
         blurCoordinates[19] = v_coordinate.xy + u_singleStepOffset * vec2(4.0, 4.0);
         blurCoordinates[20] = v_coordinate.xy + u_singleStepOffset * vec2(-2.0, -2.0);
         blurCoordinates[21] = v_coordinate.xy + u_singleStepOffset * vec2(-2.0, 2.0);
         blurCoordinates[22] = v_coordinate.xy + u_singleStepOffset * vec2(2.0, -2.0);
         blurCoordinates[23] = v_coordinate.xy + u_singleStepOffset * vec2(2.0, 2.0);

         highp float sampleColor = centralColor.g * 22.0;
         sampleColor += texture2D(u_texture, blurCoordinates[0]).g;
         sampleColor += texture2D(u_texture, blurCoordinates[1]).g;
         sampleColor += texture2D(u_texture, blurCoordinates[2]).g;
         sampleColor += texture2D(u_texture, blurCoordinates[3]).g;
         sampleColor += texture2D(u_texture, blurCoordinates[4]).g;
         sampleColor += texture2D(u_texture, blurCoordinates[5]).g;
         sampleColor += texture2D(u_texture, blurCoordinates[6]).g;
         sampleColor += texture2D(u_texture, blurCoordinates[7]).g;
         sampleColor += texture2D(u_texture, blurCoordinates[8]).g;
         sampleColor += texture2D(u_texture, blurCoordinates[9]).g;
         sampleColor += texture2D(u_texture, blurCoordinates[10]).g;
         sampleColor += texture2D(u_texture, blurCoordinates[11]).g;
         sampleColor += texture2D(u_texture, blurCoordinates[12]).g * 2.0;
         sampleColor += texture2D(u_texture, blurCoordinates[13]).g * 2.0;
         sampleColor += texture2D(u_texture, blurCoordinates[14]).g * 2.0;
         sampleColor += texture2D(u_texture, blurCoordinates[15]).g * 2.0;
         sampleColor += texture2D(u_texture, blurCoordinates[16]).g * 2.0;
         sampleColor += texture2D(u_texture, blurCoordinates[17]).g * 2.0;
         sampleColor += texture2D(u_texture, blurCoordinates[18]).g * 2.0;
         sampleColor += texture2D(u_texture, blurCoordinates[19]).g * 2.0;
         sampleColor += texture2D(u_texture, blurCoordinates[20]).g * 3.0;
         sampleColor += texture2D(u_texture, blurCoordinates[21]).g * 3.0;
         sampleColor += texture2D(u_texture, blurCoordinates[22]).g * 3.0;
         sampleColor += texture2D(u_texture, blurCoordinates[23]).g * 3.0;

         sampleColor = sampleColor / 62.0;

         highp float highPass = centralColor.g - sampleColor + 0.5;
         highPass = hardLight(highPass);
         highPass = hardLight(highPass);
         highPass = hardLight(highPass);
         highPass = hardLight(highPass);
         highPass = hardLight(highPass);

         highp float lumance = dot(centralColor, W);

         highp float alpha = pow(lumance, u_params.r);

         highp vec3 smoothColor = centralColor + (centralColor-vec3(highPass))*alpha*0.1;

         smoothColor.r = clamp(pow(smoothColor.r, u_params.g), 0.0, 1.0);
         smoothColor.g = clamp(pow(smoothColor.g, u_params.g), 0.0, 1.0);
         smoothColor.b = clamp(pow(smoothColor.b, u_params.g), 0.0, 1.0);

         highp vec3 lvse = vec3(1.0)-(vec3(1.0)-smoothColor)*(vec3(1.0)-centralColor);
         highp vec3 bianliang = max(smoothColor, centralColor);
         highp vec3 rouguang = 2.0*centralColor*smoothColor + centralColor*centralColor - 2.0*centralColor*centralColor*smoothColor;

         gl_FragColor = vec4(mix(centralColor, lvse, alpha), 1.0);
         gl_FragColor.rgb = mix(gl_FragColor.rgb, bianliang, alpha);
         gl_FragColor.rgb = mix(gl_FragColor.rgb, rouguang, u_params.b);

         highp vec3 satcolor = gl_FragColor.rgb * saturateMatrix;
         gl_FragColor.rgb = mix(gl_FragColor.rgb, satcolor, u_params.a);
         gl_FragColor.rgb = vec3(gl_FragColor.rgb + vec3(u_brightness));

     } else {
        gl_FragColor.rgb = centralColor;
    }
}
